<?php


namespace App\Http\Controllers\Back;

use Auth;
use Illuminate\Contracts\Auth\Guard;
use Illuminate\Contracts\Auth\PasswordBroker;
use Illuminate\Foundation\Auth\ResetsPasswords;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Http\Requests\Back\Account\LoginRequest;
use App\Http\Requests\Back\Account\ResetPasswordRequest;
use App\Library\Pastora;
use App\PasswordResets;
use App\User;
use Session;
use Illuminate\Support\Facades\Mail;


class AccountController extends Controller {
    use ResetsPasswords;
    protected $tokens;
    public function __construct(Guard $auth, PasswordBroker $passwords) {
        $this->auth = $auth;
        $this->passwords = $passwords;
        $this->subject = 'Recuperación de contraseña';
    }


    /**
     * [getHome description]
     * @return [type] [description]
     */

    public function getHome() {
        return view('back.account.home');
    }


    /**
     * [postLogin description]
     * @param  LoginRequest $request [description]
     * @return [type]                [description]
     */

    public function postLogin(LoginRequest $request) {
        $credentials = [
            'email' => $request->email,
            'password' => $request->password
        ];

        if (Auth::attempt($credentials)) {
            return redirect()->intended('/cms/dashboard');
        } else {
            Session::flash('loginError', true);
            Session::flash('loginMsg', 'Cuenta de correo y/o contraseña incorrectos.');
            return redirect()->intended('/cms');
        }
    }

    protected function guard() {
        return Auth::guard('clients');
    }

    public function getLogout() {
        Auth::logout();
        return redirect("/cms");
    }

    public function getRecovery() {
        return view('back.account.recovery');
    }

    public function postReset(Request $request) {
        $response = array();
        $response["status"] = false;
        $response["message"] = "";
        $new_password = Pastora::randomPassword();
        $user = User::where("email", $request->email)->first();
        if ($user != null) {
            $user->password = bcrypt($new_password);
            if ($user->save()) {
                //Enviamos el email con el resumen de la compra
                $to = $user->email;
                $subject = 'Recuperación de contraseña';
                $headers = "From: no.responder@tratamientosdeagua.com\r\n";
                $headers .= "MIME-Version: 1.0\r\n";
                $headers .= "Content-Type: text/html; charset=UTF-8\r\n";
                $message = view("mail.recover_password", ["user" => $user, "new_password" => $new_password])->render();
                //mail($to, $subject, $message, $headers);
                $email = new \SendGrid\Mail\Mail();
                $email->setFrom('no-responder@tratamientosdeagua.com', 'Water Technologies');
                $email->setSubject("Recuperación de contraseña");
                $email->addContent(
                    "text/html", $message
                );
                $email->addTo($to);
                $sendgrid = new \SendGrid('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');
                $responseGrid = $sendgrid->send($email);
                //\Mail::send("mail.recover_password", ["user" => $user, "new_password" => $new_password], function ($message) use ($user) {
                //
                //    $message->to($user->email)->subject('Recuperación de contraseña');
                //
                //});

                $response["status"] = true;
                $response["message"] = "Se ha restablecido correctamente tu contraseña de acceso. Te hemos enviado un correo electrónico con tu nueva contraseña.";
                $response["url"] = url("cms");
            } else {
                $response["status"] = false;
                $response["message"] = "El correo electrónico no se encuentra registrado, intentalo nuevamente.";
            }
        } else {
            $response["status"] = false;
            $response["message"] = "El correo electrónico no se encuentra registrado, intentalo nuevamente.";
        }
        return response()->json($response);
    }
}