<?php



namespace App\Http\Controllers\Back\Dashboard\Catalog;



use Illuminate\Http\Request;

use Illuminate\Support\Facades\DB;

use App\Brand;

use App\Http\Controllers\Controller;

use App\Http\Requests\Back\Dashboard\Catalog\BrandRequest;

use App\Library\URI;



class BrandController extends Controller

{



    /**

     * Display a listing of the resource.

     *

     * @return \Illuminate\Http\Response

     */

    public function index()

    {

        $brands = Brand::orderBy('name', 'ASC')->get();



        return view('back.catalog.brand.index', ["data" => $brands, "permitions" => URI::checkPermitions()]);

    }



    /**

     * Show the form for creating a new resource.

     *

     * @return \Illuminate\Http\Response

     */

    public function create()

    {

        return view('back.catalog.brand.create');

    }



    /**

     * Store a newly created resource in storage.

     *

     * @param  \Illuminate\Http\Request  $request

     * @return \Illuminate\Http\Response

     */

    public function store(BrandRequest $request)

    {

        $response = array('status' => false);



        DB::beginTransaction();



        $brand = new Brand();

        $brand->name = $request->nombre;



        if($brand->save()) {



            DB::commit();



            $response['status'] = true;

            $response["message"] = "Los datos se han almacenado correctamente.";

            $response['url'] = route('brand_index');



        }else{

            DB::rollBack();

            $response['status'] = false;

            $response["message"] = "No hemos podido almacenar los datos, intentalo nuevamente.";

        }



        return response()->json($response);

    }



    /**

     * Show the form for editing the specified resource.

     *

     * @param  int  $id

     * @return \Illuminate\Http\Response

     */

    public function edit($id)

    {

        $brand = Brand::findOrFail(base64_decode($id));



        return view('back.catalog.brand.edit', ["data" => $brand]);

    }



    /**

     * Update the specified resource in storage.

     *

     * @param  \Illuminate\Http\Request  $request

     * @param  int  $id

     * @return \Illuminate\Http\Response

     */

    public function update(BrandRequest $request, $id)

    {

        $response = array('status' => false);



        DB::beginTransaction();



        $brand = Brand::find($id);

        $brand->name = $request->nombre;



        if($brand->save()) {



            DB::commit();



            $response['status'] = true;

            $response["message"] = "Los datos se han editado correctamente.";

            $response['url'] = route('brand_index');



        }else{

            DB::rollBack();

            $response['status'] = false;

            $response["message"] = "No hemos podido edutar los datos, intentalo nuevamente.";

        }



        return response()->json($response);

    }



    /**

     * Remove the specified resource from storage.

     *

     * @param  int  $id

     * @return \Illuminate\Http\Response

     */

    public function destroy($id)

    {



        $response = array(

            'status' => true,

            'message' => 'El usuario ha sido eliminado correctamente.',

            'url' => route('user_index'),

        );



        $data = Brand::find(base64_decode($id));

        if ($data->products->count() == 0) {



            if($data != null) {

                if($data->delete()) {



                    $response = [

                        'message' => 'Se ha eliminado el registro correctamente.',

                        'status' => true,

                        'url' => route('brand_index')

                    ];

                }

            } else {

                $response = [

                    'message' => 'No hemos podido eliminar el registro, intentalo nuevamente.',

                    'status' => false

                ];

            }



        } else{

            $response = [

                'message' => 'No se puede eliminar la marca. La marca esta relacionada a alguún producto.',

                'status' => false

            ];

        }



        return response()->json($response);

    }



}