<?php



namespace App\Http\Controllers\Back\Dashboard\Catalog;



use Carbon\Carbon;

use Illuminate\Http\Request;

use Illuminate\Support\Facades\DB;

use App\Brand;

use App\Http\Controllers\Controller;

use App\Http\Requests\Back\Dashboard\Catalog\EditProductRequest;

use App\Http\Requests\Back\Dashboard\Catalog\ProductRequest;

use App\Library\Pastora;

use App\Library\URI;

use App\Product;

use App\ProductCategory;



class ProductController extends Controller

{



    /**

     * Display a listing of the resource.

     *

     * @return \Illuminate\Http\Response

     */

    public function index()

    {

        $brands = Product::orderBy('name', 'ASC')->get();



        return view('back.catalog.product.index', ["data" => $brands, "permitions" => URI::checkPermitions()]);

    }



    /**

     * Show the form for creating a new resource.

     *

     * @return \Illuminate\Http\Response

     */

    public function create()

    {



        $brands = Brand::orderBy('name', 'asc')->get();

        $categories = ProductCategory::orderBy('name', 'asc')->get();

        $products_related = Product::get();



        return view('back.catalog.product.create', compact('brands', 'categories', 'products_related'));

    }



    /**

     * Store a newly created resource in storage.

     *

     * @param  \Illuminate\Http\Request  $request

     * @return \Illuminate\Http\Response

     */

    public function store(ProductRequest $request)

    {



        $response = array('status' => false);



        DB::beginTransaction();



        $product = new Product();

        $product->name = $request->nombre;

        $product->product_code = $request->codigo_producto;

        $product->application_type = $request->tipo_aplicacion;

        $product->description = $request->descripcion;

        $product->keywords = $request->palabrasClave;

        $product->data_sheet_description = $request->fichaTecnicaDescripcion;

        $product->brand_id = $request->marca;

        $product->product_category_id = $request->categoria;

        if ( is_array($request->products_related) ) {
            $product->products_related = implode(",", $request->products_related);
        } else {
            $product->products_related = "";
        }

        $product->producto_principal = (($request->producto_principal == "on") ? true : false);



        //Agregamos los id de referencia

        $category = ProductCategory::find($request->categoria);

        $product->id_referencia_category = Pastora::cleanStr($category->name);

        $product->id_referencia_name = Pastora::cleanStr($request->nombre);



        $product->save();





        $path = Pastora::createPathImage($product->id);

        if( !is_dir($path) ){

            mkdir($path, 0777, true);

        }





        //Files

        if( $request->has("fichaTecnica") ) {

            $fileName = Pastora::createNameImage($path, $request->file('fichaTecnica'));

            $request->file('fichaTecnica')->move($path, $fileName);

            $product->data_sheet = $path . $fileName;

        }



        if( $request->has("manualDeInstalacion") ) {

            $fileName = Pastora::createNameImage($path, $request->file('manualDeInstalacion'));

            $request->file('manualDeInstalacion')->move($path, $fileName);

            $product->installation_manual = $path . $fileName;

        }



        $product->price = 0;

        $product->discount = 0;

        $product->weight = $request->peso;

        $product->quantity = 0;

        $product->minimum = 0;

        $product->video = $request->video;



        //Images

        if( $request->has("imagenUno") ) {

            $fileName = Pastora::createNameImage($path, $request->file('imagenUno'));

            $request->file('imagenUno')->move($path, $fileName);

            $product->photo_one = $path . $fileName;

        }



        if( $request->has("imagenDos") ) {

            $fileName = Pastora::createNameImage($path, $request->file('imagenDos'));

            $request->file('imagenDos')->move($path, $fileName);

            $product->photo_two = $path . $fileName;

        }



        if( $request->has("imagenTres") ) {

            $fileName = Pastora::createNameImage($path, $request->file('imagenTres'));

            $request->file('imagenTres')->move($path, $fileName);

            $product->photo_three = $path . $fileName;

        }



        $product->benefits = $request->beneficios;

        $product->uses = $request->usos;

        $product->precautions = $request->precauciones;

        $product->delivery_conditions = $request->condiciones;



        $product->height = $request->alto;

        $product->width = $request->ancho;

        $product->length = $request->largo;



        if($product->save()) {



            DB::commit();



            $response['status'] = true;

            $response["message"] = "Los datos se han almacenado correctamente.";

            $response['url'] = route('products_index');



        }else{

            DB::rollBack();

            $response['status'] = false;

            $response["message"] = "No hemos podido almacenar los datos, intentalo nuevamente.";

        }



        return response()->json($response);

    }



    /**

     * Show the form for editing the specified resource.

     *

     * @param  int  $id

     * @return \Illuminate\Http\Response

     */

    public function edit($id)

    {

        $product = Product::find(base64_decode($id));

        $brands = Brand::orderBy('name', 'asc')->get();

        $categories = ProductCategory::orderBy('name', 'asc')->get();



        $produt_already_related = array();

        foreach (explode(",", $product->products_related) as $item){

            $produt_already_related[] = $item;

        }



        $products_related = Product::where("id", "!=", base64_decode($id))->get();



        return view('back.catalog.product.edit', compact('product', 'brands', 'categories', 'products_related', 'produt_already_related'));

    }



    /**

     * Update the specified resource in storage.

     *

     * @param  \Illuminate\Http\Request  $request

     * @param  int  $id

     * @return \Illuminate\Http\Response

     */

    public function update(EditProductRequest $request, $id)

    {

        $response = array('status' => false);

        DB::beginTransaction();

        $product = Product::findOrFail($id);

        $product->name = $request->nombre;

        $product->product_code = $request->codigo_producto;

        $product->application_type = $request->tipo_aplicacion;

        $product->description = $request->descripcion;

        $product->keywords = $request->palabrasClave;

        $product->data_sheet_description = $request->fichaTecnicaDescripcion;

        $product->brand_id = $request->marca;

        $product->product_category_id = $request->categoria;

        $product->products_related = implode(",", $request->products_related);

        $product->producto_principal = (($request->producto_principal == "on") ? true : false);





        //Agregamos los id de referencia

        $category = ProductCategory::find($request->categoria);

        $product->id_referencia_category = Pastora::cleanStr($category->name);

        $product->id_referencia_name = Pastora::cleanStr($request->nombre);





        //$path = Pastora::createPathImage($request->categoria, $request->marca);

        $path = Pastora::createPathImage($id);

        if( !is_dir($path) ){

            mkdir($path, 0777, true);

        }



        //Files

        if ( $request->has('fichaTecnica') && $request->fichaTecnica != null ) {

            $fileName = Pastora::createNameImage($path, $request->file('fichaTecnica'));

            $request->file('fichaTecnica')->move($path, $fileName);

            $product->data_sheet = $path.$fileName;

        }



        if ($request->has('manualDeInstalacion') && $request->manualDeInstalacion != null ) {

            $fileName = Pastora::createNameImage($path, $request->file('manualDeInstalacion'));

            $request->file('manualDeInstalacion')->move($path, $fileName);

            $product->installation_manual = $path.$fileName;

        }



        //$product->price = $request->precio;

        //$product->discount = $request->descuento;

        $product->weight = $request->peso;

        //$product->quantity = $request->cantidad;

        //$product->minimum = $request->minimo;

        $product->video = $request->video;



        //Images

        if ( $request->has('imagenUno') && $request->imagenUno != null )  {

            $fileName = Pastora::createNameImage($path, $request->file('imagenUno'));

            $request->file('imagenUno')->move($path, $fileName);

            $product->photo_one = $path.$fileName;

        }

//        else{

//            if(file_exists($product->photo_one)) {

//                $file_name = explode("/", $product->photo_one);

//                rename($product->photo_one, $path . "" . $file_name[count($file_name) - 1]);

//                $product->photo_one = $path . $file_name[count($file_name) - 1];

//            }

//        }



        if ( $request->has('imagenDos') && $request->imagenDos != null ) {

            $fileName = Pastora::createNameImage($path, $request->file('imagenDos'));

            $request->file('imagenDos')->move($path, $fileName);

            $product->photo_two = $path.$fileName;

        }

//        else{

//            if(file_exists($product->photo_two)){

//                $file_name = explode("/", $product->photo_two);

//                rename($product->photo_two, $path . "" . $file_name[count($file_name) - 1]);

//                $product->photo_two = $path . $file_name[count($file_name) - 1];

//            }

//        }



        if ($request->has('imagenTres') && $request->imagenTres != null ) {

            $fileName = Pastora::createNameImage($path, $request->file('imagenTres'));

            $request->file('imagenTres')->move($path, $fileName);

            $product->photo_three = $path.$fileName;

        }

//        else{

//            if(file_exists($product->photo_three)) {

//                $file_name = explode("/", $product->photo_three);

//                rename($product->photo_three, $path . "" . $file_name[count($file_name) - 1]);

//                $product->photo_three = $path . $file_name[count($file_name) - 1];

//            }

//        }



        $product->benefits = $request->beneficios;

        $product->uses = $request->usos;

        $product->precautions = $request->precauciones;

        $product->delivery_conditions = $request->condiciones;



        $product->height = $request->alto;

        $product->width = $request->ancho;

        $product->length = $request->largo;



        if($product->save()) {



            DB::commit();



            $response['status'] = true;

            $response["message"] = "Los datos se han editado correctamente.";

            $response['url'] = route('products_index');



        }else{

            DB::rollBack();

            $response['status'] = false;

            $response["message"] = "No hemos podido editar los datos, intentalo nuevamente.";

        }



        return response()->json($response);

    }



    /**

     * Remove the specified resource from storage.

     *

     * @param  int  $id

     * @return \Illuminate\Http\Response

     */

    public function destroy($id)

    {



        $response = array(

            'status' => true,

            'message' => 'El usuario ha sido eliminado correctamente.',

            'url' => route('user_index'),

        );



        $data = Product::find(base64_decode($id));

        if($data != null) {

            if($data->delete()) {



//                if( file_exists($data->data_sheet) ){

//                    unlink($data->data_sheet);

//                }

//                if( file_exists($data->installation_manual) ){

//                    unlink($data->installation_manual);

//                }

//                if( file_exists($data->photo_one) ){

//                    unlink($data->photo_one);

//                }

//                if( file_exists($data->photo_two) ){

//                    unlink($data->photo_two);

//                }

//                if( file_exists($data->photo_three) ){

//                    unlink($data->photo_three);

//                }



                $response = [

                    'message' => 'Se ha eliminado el registro correctamente.',

                    'status' => true,

                    'url' => route('products_index')

                ];

            }

        } else {

            $response = [

                'message' => 'No hemos podido eliminar el registro, intentalo nuevamente.',

                'status' => false

            ];

        }



        return response()->json($response);

    }

}

