<?php 

namespace App\Library;

use Auth;
use GuzzleHttp\Psr7\Request;
use Illuminate\Support\Facades\Mail;
use App\Brand;
use App\Order;
use App\Product;
use App\ProductCategory;
use App\Reservation;
use App\ShippingPrice;
use App\SystemModule;
use App\User;
use Permission;
use Session;


class Pastora {
    public static function in_relation($object, $field_id, $field) {
        if (!is_null($object)) {
            if (!is_null($object->$field_id)) {
                if (!is_null($object->$field_id->$field)) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * Metodo que devuelve verdadero si una propiedad se encuentra en un arreglo
     *
     * @return Boolean
     */

    public static function in_object($value, $object){
        if (is_object($object)) {
            foreach ($object as $key => $item) {
                if ($value == $key) return true;
            }
        }
        return false;
    }

    /**
     * Metodo que devuelve los permisos que tiene el usuario
     *
     * @return JSON
     */

    public static function userProfile()

    {


        return User::find(Auth::user()->id)->userProfile->permits;

    }


    /**
     * Metodo que convierte un Json a Array
     *
     * @return Array
     */

    public static function jsonToArray($json)

    {

        $json = (array)json_decode($json);

        $response = [];

        foreach ($json as $key => $value) {

            $response[(int)$key] = (int)$value;

        }

        return $response;

    }


    /**
     * This method convert the array to object
     *
     * @param $array array to convert to object
     * @return mixed return the array on object
     */

    public static function arrayToObject($array)
    {

        return json_decode(json_encode($array));

    }


    /**
     *
     *

     */

    public function strongMeter($password)

    {

        $rank = 0;

        $rank += (strlen(trim($password)) > 7) ? 5 : 0;

        $rank += (strlen(trim($password)) > 11 && strlen(trim($password)) < 16) ? 10 : 0;

        $rank += (preg_match("/([A-Z])/", $password)) ? 10 : 0;

        $rank += (preg_match("/([0-9])/", $password)) ? 10 : 0;

    }


    /**
     * Metodo que construye el arbol de modulos del sistema
     *
     * @var $parent integer. ID del papa del módulo
     *
     * @return array
     */

    public static function moduleTree($parent = 0)

    {


        $response = [];


        foreach (SystemModule::where('parent', '=', $parent)->orderBy('order')->get() as $module) {


            $response[] = [

                'id' => $module->id,

                'name' => $module->name,

                'url' => $module->url,

                'icon' => $module->icon,

                'parent_as_child' => $module->parent_as_child,

                'child' => self::moduleTree($module->id)

            ];


        }


        return $response;


    }


    /**
     *
     *

     */

    public static function cleanPhone($phone)

    {

        $mapper = [

            '/',

            '(',

            ')',

            '-',

            ' '

        ];


        foreach ($mapper as $key => $value) {


            $phone = str_replace($value, "", $phone);


        }


        return $phone;

    }


    public static function randomPassword()
    {

        $alphabet = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890';

        $pass = array(); //remember to declare $pass as an array

        $alphaLength = strlen($alphabet) - 1; //put the length -1 in cache

        for ($i = 0; $i < 6; $i++) {

            $n = rand(0, $alphaLength);

            $pass[] = $alphabet[$n];

        }


        return implode($pass); //turn the array into a string

    }


    public function getRandomWord($len = 5)
    {

        $word = array_merge(range('0', '9'), range('A', 'Z'));

        shuffle($word);

        return substr(implode($word), 0, $len);

    }


    //PAYMENTS CONEKTA

    public static function create_customer($name, $email, $phone, $token_id)
    {


        $response = array();

        $response["error"] = false;

        $response["message"] = "";

        $response["customer"] = "";


        \Conekta\Conekta::setApiKey("key_FqiqnZjaRFMPJ2rqtnjcMg");

        \Conekta\Conekta::setApiVersion("2.0.0");


        try {


            $customer = \Conekta\Customer::create(

                array(

                    "name" => $name,

                    "email" => $email,

                    "phone" => $phone,

                    "payment_sources" => array(

                        array(

                            "type" => "card",

                            "token_id" => $token_id,

                        )

                    )//payment_sources

                )//customer

            );


            $response["error"] = false;

            $response["message"] = "";

            $response["customer"] = $customer;


        } catch (\Conekta\ProccessingError $error) {

            $response["error"] = true;

            $response["message"] = $error->message;

        } catch (\Conekta\ParameterValidationError $error) {

            $response["error"] = true;

            $response["message"] = $error->message;

        } catch (\Conekta\Handler $error) {

            $response["error"] = true;

            $response["message"] = $error->message;

        }


        return $response;

    }


    public static function make_payment($concepto, $cantidad_cobrar, $customer)
    {


        $cantidad_cobrar = ($cantidad_cobrar * 100);

        $cantidad_cobrar = explode(".", $cantidad_cobrar);


        $response = array();

        $response["error"] = false;

        $response["message"] = "";

        $response["customer"] = "";


        \Conekta\Conekta::setApiKey("key_FqiqnZjaRFMPJ2rqtnjcMg");

        \Conekta\Conekta::setApiVersion("2.0.0");


        try {

            $order = \Conekta\Order::create(

                array(

                    "line_items" => array(

                        array(

                            "name" => $concepto,

                            "unit_price" => $cantidad_cobrar[0],

                            "quantity" => 1

                        )

                    ),

//                    "shipping_lines" => array(

//                        array(

//                            "amount" => 1500,

//                            "carrier" => "FEDEX"

//                        )

//                    ), //shipping_lines - physical goods only

                    "currency" => "MXN",

                    "customer_info" => array(

                        "customer_id" => $customer->id

                    ), //customer_info

//                    "shipping_contact" => array(

//                        "address" => array(

//                            "street1" => "Calle 123, int 2",

//                            "postal_code" => "06100",

//                            "country" => "MX"

//                        )//address

//                    ), //shipping_contact - required only for physical goods

//                    "metadata" => array("reference" => "12987324097", "more_info" => "lalalalala"),

                    "charges" => array(

                        array(

                            "payment_method" => array(

                                "type" => "default"

                            ) //payment_method - use customer's default - a card

                            //to charge a card, different from the default,

                            //you can indicate the card's source_id as shown in the Retry Card Section

                        ) //first charge

                    ) //charges

                )//order

            );


            $response["error"] = false;

            $response["message"] = "";

            $response["order"] = $order;


        } catch (\Conekta\ProcessingError $error) {

            $response["error"] = true;

            $response["message"] = $error->message;

        } catch (\Conekta\ParameterValidationError $error) {

            $response["error"] = true;

            $response["message"] = $error->message;

        } catch (\Conekta\Handler $error) {

            $response["error"] = true;

            $response["message"] = $error->message;

        }


        return $response;

    }


    public static function make_order_spei($concepto, $cantidad_cobrar, $name, $email, $phone)
    {


        $cantidad_cobrar = ($cantidad_cobrar * 100);

        $cantidad_cobrar = explode(".", $cantidad_cobrar);


        $response = array();

        $response["error"] = false;

        $response["message"] = "";

        $response["customer"] = "";


        \Conekta\Conekta::setApiKey("key_FqiqnZjaRFMPJ2rqtnjcMg");

        \Conekta\Conekta::setApiVersion("2.0.0");


        try {

            $order = \Conekta\Order::create(

                array(

                    "line_items" => array(

                        array(

                            "name" => $concepto,

                            "unit_price" => $cantidad_cobrar[0],

                            "quantity" => 1

                        )//first line_item

                    ), //line_items

//                    "shipping_lines" => array(

//                        array(

//                            "amount" => 1500,

//                            "carrier" => "FEDEX"

//                        )

//                    ), //shipping_lines - physical goods only

                    "currency" => "MXN",

                    "customer_info" => array(

                        "name" => $name,

                        "email" => $email,

                        "phone" => $phone,

                    ), //customer_info

//                    "shipping_contact" => array(

//                        "address" => array(

//                            "street1" => "Calle 123, int 2",

//                            "postal_code" => "06100",

//                            "country" => "MX"

//                        )//address

//                    ), //shipping_contact - required only for physical goods

                    "charges" => array(

                        array(

                            "payment_method" => array(

                                "type" => "spei"

                            )//payment_method

                        ) //first charge

                    ) //charges

                )//order

            );


            $response["error"] = false;

            $response["message"] = "";

            $response["order"] = $order;


        } catch (\Conekta\ParameterValidationError $error) {

            $response["error"] = true;

            $response["message"] = $error->message;

        } catch (\Conekta\Handler $error) {

            $response["error"] = true;

            $response["message"] = $error->message;

        }


        return $response;

    }


    public static function make_order_oxxo($concepto, $cantidad_cobrar, $name, $email, $phone)
    {


        $cantidad_cobrar = ($cantidad_cobrar * 100);

        $cantidad_cobrar = explode(".", $cantidad_cobrar);


        $response = array();

        $response["error"] = false;

        $response["message"] = "";

        $response["customer"] = "";


        \Conekta\Conekta::setApiKey("key_FqiqnZjaRFMPJ2rqtnjcMg");

        \Conekta\Conekta::setApiVersion("2.0.0");


        try {


            $order = \Conekta\Order::create(

                array(

                    "line_items" => array(

                        array(

                            "name" => $concepto,

                            "unit_price" => $cantidad_cobrar[0],

                            "quantity" => 1

                        )//first line_item

                    ), //line_items

//                    "shipping_lines" => array(

//                        array(

//                            "amount" => 1500,

//                            "carrier" => "FEDEX"

//                        )

//                    ), //shipping_lines - physical goods only

                    "currency" => "MXN",

                    "customer_info" => array(

                        "name" => $name,

                        "email" => $email,

                        "phone" => $phone

                    ), //customer_info

//                      "shipping_contact" => array(

//                        "address" => array(

//                          "street1" => "Calle 123, int 2",

//                          "postal_code" => "06100",

//                          "country" => "MX"

//                        )//address

//                      ), //shipping_contact - required only for physical goods

                    "charges" => array(

                        array(

                            "payment_method" => array(

                                "type" => "oxxo_cash"

                            )//payment_method

                        ) //first charge

                    ) //charges

                )//order

            );


            $response["error"] = false;

            $response["message"] = "";

            $response["order"] = $order;


        } catch (\Conekta\ParameterValidationError $error) {

            $response["error"] = true;

            $response["message"] = $error->message;

        } catch (\Conekta\Handler $error) {

            $response["error"] = true;

            $response["message"] = $error->message;

        }


        return $response;

    }


    public static function make_sku()
    {


        $reservation = null;

        $sku = "";


        do {


            $sku = rand(1000, 9999) . "-" . chr(rand(65, 90)) . chr(rand(65, 90));

            $reservation = Reservation::where("SKU", $sku)->count();


        } while ($reservation >= 1);


        return $sku;

    }





    //EMAIL WITH RESERVATION WITH STATUS

    //Es el correo que se envía cuando el usuario hace el pago. Cambia el formato dependiendo si es Tarjeta, SPEI o OXXO PAY

    public static function email_reservation_with_status($client, $reservation, $registed_room)
    {


        Mail::send('mail.payment_or_baucher_first_step', array("client" => $client, "reservation" => $reservation, "room" => $registed_room), function ($message) use ($client) {


            //remitente

            $message->from("noresponder@ikaan.com.mx", "Ikaan reservaciones");


            //asunto

            $message->subject("Reservación Ikaan");


            //receptor

            //$message->to($client->email, $client->nombre_completo);

            $message->to("jair.lomas@metodika.mx", "Jair Lomas");


        });


    }


    public static function sendPushNotification($notification)
    {


        $API_ACCESS_KEY = env('SERVER_KEY_FIREBASE');


        $singleID = $notification->RESPONSE["UUID"];


        $instructions = array();

        $instructions["TEXTO_NOTIFICACION"] = $notification->RESPONSE["TEXTO_NOTIFICACION"];


        $fcmMsg = array(

            'body' => $notification->RESPONSE["TEXTO_NOTIFICACION"],

            'title' => $notification->RESPONSE["TITLE"],

            'subtitle' => "Recsol",

            'sound' => "default",

        );


        $fcmFields = array(

            'to' => $singleID,

            'priority' => 'high',

            'data' => $instructions,

            'notification' => $fcmMsg

        );


        $headers = array(

            'Authorization: key=' . $API_ACCESS_KEY,

            'Content-Type: application/json'

        );


        $ch = curl_init();

        curl_setopt($ch, CURLOPT_URL, 'https://fcm.googleapis.com/fcm/send');

        curl_setopt($ch, CURLOPT_POST, true);

        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);

        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($fcmFields));

        $result = curl_exec($ch);

        curl_close($ch);


        $result = json_decode($result);


        return $result;

    }


    public static function autocomplete_array($obj, $colum)
    {


        $obj = $obj->groupBy($colum)->get()->toArray();

        $tags = "";

        foreach ($obj as $item) {

            $tags .= $item[$colum] . ",";

        }


        return $tags;

    }


    public static function createPathImage($id)
    {

//        $category       = ProductCategory::find($category_id);
//        $brand          = Brand::find($brand_id);
//
//        $category = self::cleanStr($category->name);
//        $brand    = self::cleanStr($brand->name);
//
//        $path = "productos-water-technologies/".$category."/".$brand."/";

        $path = "productos-water-technologies/" . $id . "/";

        return $path;
    }


    public static function createNameImage($path, $image_object)
    {

        $image_name = str_replace("." . $image_object->getClientOriginalExtension(), "", $image_object->getClientOriginalName());
        $image_name = self::cleanStr($image_name);
        $image_name_created = $image_name . "." . $image_object->getClientOriginalExtension();

        if (file_exists($path . $image_name_created)) {
            $image_name_created = $image_name . "_" . rand(100, 1000) . "_." . $image_object->getClientOriginalExtension();
        }

        return $image_name_created;
    }


    public static function cleanStr($string)
    {

        $string = trim($string);


        $string = str_replace(
            array('á', 'à', 'ä', 'â', 'ª', 'Á', 'À', 'Â', 'Ä'),
            array('a', 'a', 'a', 'a', 'a', 'A', 'A', 'A', 'A'),
            $string
        );

        $string = str_replace(
            array('é', 'è', 'ë', 'ê', 'É', 'È', 'Ê', 'Ë'),
            array('e', 'e', 'e', 'e', 'E', 'E', 'E', 'E'),
            $string
        );

        $string = str_replace(
            array('í', 'ì', 'ï', 'î', 'Í', 'Ì', 'Ï', 'Î'),
            array('i', 'i', 'i', 'i', 'I', 'I', 'I', 'I'),
            $string
        );

        $string = str_replace(
            array('ó', 'ò', 'ö', 'ô', 'Ó', 'Ò', 'Ö', 'Ô'),
            array('o', 'o', 'o', 'o', 'O', 'O', 'O', 'O'),
            $string
        );

        $string = str_replace(
            array('ú', 'ù', 'ü', 'û', 'Ú', 'Ù', 'Û', 'Ü'),
            array('u', 'u', 'u', 'u', 'U', 'U', 'U', 'U'),
            $string
        );

        $string = str_replace(
            array('ñ', 'Ñ', 'ç', 'Ç'),
            array('n', 'N', 'c', 'C',),
            $string
        );


        // Replaces all spaces with hyphens.
        $string = str_replace(' ', '-', $string);

        // Removes special chars.
        $string = preg_replace('/[^A-Za-z0-9\-]/', '', $string);
        // Replaces multiple hyphens with single one.
        $string = preg_replace('/-+/', '-', $string);

        return $string;
    }


    public static function getSubtotalInCart()
    {

        $products = self::formatProductCart();

        $total = 0;
        foreach ($products as $product) {
            $total += $product["total"];
        }

        $total = number_format(floor($total * pow(10, 2)) / pow(10, 2), 2, ".", "");

        return $total;
    }


    public static function getIvaInCart()
    {

        if (session()->has("correo_electronico")) {
            if (session()->get("correo_electronico") == "jair.lomas@metodika.mx" || session()->get("correo_electronico") == "luz.garcia@metodika.mx") {
                return 1;
            }
        }

        $total_in_cart = self::getSubtotalInCart();
        $envio = self::getEnvioInCart();

        if ($total_in_cart != 0 && $envio["total_shipping"] != 0) {
            $total_calcular_iva = $total_in_cart + $envio["total_shipping"];
            $iva = ($total_calcular_iva * .16);
        } else {
            $iva = 0;
        }

        $iva = number_format(floor($iva * pow(10, 2)) / pow(10, 2), 2, ".", "");

        return $iva;
    }


    public static function getEnvioInCart()
    {

        if (session()->has("correo_electronico")) {
            if (session()->get("correo_electronico") == "jair.lomas@metodika.mx" || session()->get("correo_electronico") == "luz.garcia@metodika.mx") {
                $shipping_price = array("price_type" => "A",
                    "0_2" => (1),
                    "21_4" => (1),
                    "41_6" => (1),
                    "61_8" => (1));

                return self::shipping_cost($shipping_price);
            }
        }

        if (session()->has("state") && session()->has("destiny") && session()->has("shipping_type")) {

            if (session()->get("state") != "" && session()->get("destiny") != "" && session()->get("shipping_type") != "") {

                //ENVIO STANDAR O EXPRESS
                if (session()->get("shipping_type") == "envio_standar") {

                    //CASTORES
                    $state_shipping = session()->get("state");
                    $destiny_shipping = session()->get("destiny");

                    //Obtenemos la zona para calcular el precio
                    $shipping_price = ShippingPrice::where("state", $state_shipping)->where("destiny", $destiny_shipping)->first();

                    $shipping_price = array("price_type" => $shipping_price->price_type,
                        "0_2" => ($shipping_price->ordinary * 1),
                        "21_4" => ($shipping_price->ordinary * 2),
                        "41_6" => ($shipping_price->ordinary * 3),
                        "61_8" => ($shipping_price->ordinary * 4));

                    return self::shipping_cost($shipping_price);

                } else {

                    //DHL
                    $state_shipping = session()->get("state");
                    $destiny_shipping = session()->get("destiny");

                    //Obtenemos la zona para calcular el precio
                    $shipping_price = ShippingPrice::where("state", $state_shipping)->where("destiny", $destiny_shipping)->first();
                    $shipping_price = $shipping_price->prices->toArray();

                    return self::shipping_cost($shipping_price);

                }

            } else {
                return array("total_shipping" => 0, "packages" => 0, "volumentric_weight" => 0);
            }

        } else {
            return array("total_shipping" => 0, "packages" => 0, "volumentric_weight" => 0);
        }

    }


    public static function getTotalInCart()
    {

        $subtotal = self::getSubtotalInCart();
        $iva = self::getIvaInCart();
        $envio = self::getEnvioInCart();

        $total = $subtotal + $iva + $envio["total_shipping"];

        return $total;
    }


    public static function getQuantityProductsInCart()
    {

        $products = self::formatProductCart();

        $total_in_cart = 0;
        foreach ($products as $product) {
            $total_in_cart += $product["total_to_buy"];
        }

        return $total_in_cart;
    }


    public static function formatProductCart()
    {

        $products = session()->all();
        $product_list = array();

        //Primero iteramos tods los productos, hay veces que la sesion traer basura por lo que tenemos que traer que las primeras 7 letras sea product
        foreach ($products as $key => $total_to_buy) {

            if (substr($key, 0, 8) == "product_") {
                $product_id = substr($key, 8, strlen($key) + 1);

                $product_object = Product::find($product_id);

                $discount = 0;

                //Si hay descuento hacemos el calculo
                if ($product_object->discount > 0) {
                    $discount = ($product_object->discount / 100) * $product_object->price;
                    $real_price = $product_object->price - $discount;
                } else {
                    $real_price = $product_object->price;
                }

                $real_price = number_format(floor($real_price * pow(10, 2)) / pow(10, 2), 2);
                $real_price = (float)str_replace(",", "", $real_price);

                $total = $real_price * $total_to_buy;

                $product_list[] = array(
                    "product" => $product_object,
                    "original_price" => $product_object->price,
                    "percent_discount" => $product_object->discount,
                    "price_with_discount" => $real_price,
                    "total_to_buy" => $total_to_buy,
                    "total" => $total,
                );
            }

        }


        return $product_list;
    }


    public static function generateOrderId()
    {

        $reservation = null;
        $sku = "";

        do {

            $sku = "WT" . rand(1000, 9999) . "-" . date("dmYHis");
            $reservation = Order::where("sku", $sku)->count();

        } while ($reservation >= 1);

        return $sku;
    }


    public static function get_total_volumetric_weight()
    {

        $products = self::formatProductCart();

        $total_volumentric_weight = 0;
        foreach ($products as $product) {
            $total_volumentric_weight += $product["product"]->weight_volumetric * $product["total_to_buy"];
        }

        return $total_volumentric_weight;
    }


    public static function shipping_cost($shipping_price)
    {

        $volumentric_weight = self::get_total_volumetric_weight();
        $total_volumentric_weight = $volumentric_weight;

        $packages = array();
        $total_shipp = 0;

        do {

            $routine_enter_package = false;

            // Mayor a 8
            if (($total_volumentric_weight > 8) && !$routine_enter_package) {
                $total_volumentric_weight -= 8;
                $packages[] = "61_8";
                $routine_enter_package = true;
                $total_shipp += $shipping_price["61_8"];
            }

            // Entre a 6.1 y 8
            if (($total_volumentric_weight >= 6.1 && $total_volumentric_weight <= 8) && !$routine_enter_package) {
                $total_volumentric_weight -= 8;
                $packages[] = "61_8";
                $routine_enter_package = true;
                $total_shipp += $shipping_price["61_8"];
            }

            // Entre a 4.1 y 6
            if (($total_volumentric_weight >= 4.1 && $total_volumentric_weight <= 6) && !$routine_enter_package) {
                $total_volumentric_weight -= 6;
                $packages[] = "41_6";
                $routine_enter_package = true;
                $total_shipp += $shipping_price["41_6"];
            }

            // Entre a 2.1 y 4
            if (($total_volumentric_weight >= 2.1 && $total_volumentric_weight <= 4) && !$routine_enter_package) {
                $total_volumentric_weight -= 4;
                $packages[] = "21_4";
                $routine_enter_package = true;
                $total_shipp += $shipping_price["21_4"];
            }

            // Entre a 0 y 2
            if (($total_volumentric_weight >= 0 && $total_volumentric_weight <= 2) && !$routine_enter_package) {
                $total_volumentric_weight -= 2;
                $packages[] = "0_2";
                $routine_enter_package = true;
                $total_shipp += $shipping_price["0_2"];
            }


        } while ($total_volumentric_weight > 0);

        return array("total_shipping" => $total_shipp, "packages" => $packages, "volumentric_weight" => $volumentric_weight);
    }

}