<?php

use Illuminate\Support\Facades\Route;
use Illuminate\Support\Facades\Auth;
use App\Http\Controllers\Back\AccountController;
use App\Http\Controllers\Back\Dashboard\DashboardController;
use App\Http\Controllers\Back\Dashboard\System\UserController;
use App\Http\Controllers\Back\Dashboard\System\ProfileController;
use App\Http\Controllers\Back\Dashboard\Productos\productosSeccionesController;
use App\Http\Controllers\Back\Dashboard\Home\bannersController;

/*
|--------------------------------------------------------------------------
| Web Routes
|--------------------------------------------------------------------------
|
| Here is where you can register web routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| contains the "web" middleware group. Now create something great!
|
*/

//WEB
Route::group(['prefix' => 'cms'], function () {
    Route::get('/', [AccountController::class, 'getHome'])->name('login');
    Route::group(['prefix' => 'account'], function () {
        Route::get('logout', [AccountController::class, 'getLogout'])->name('admin_logout');
        Route::post('login', [AccountController::class, 'postLogin'])->name('admin_login');
        Route::post('password/reset', [AccountController::class, 'postReset'])->name('password.reset');
    });
    Route::group(['middleware' => ['auth']], function () {
        Route::group(['prefix' => 'dashboard'], function () {
            Route::get('/', [DashboardController::class, 'index']);
            Route::group(['prefix' => 'system'], function () {
                Route::group(['prefix' => 'user'], function () {
                    Route::get('/', [UserController::class, 'index'])->name('user_index');
                    Route::get('create', [UserController::class, 'create'])->name('user_create_get');
                    Route::post('store', [UserController::class, 'store'])->name('user_create');
                    Route::get('edit/{id}', [UserController::class, 'edit'])->name('user_edit');
                    Route::post('update/{id}', [UserController::class, 'update'])->name('user_update');
                    Route::get('delete/{id}', [UserController::class, 'destroy'])->name('user_delete');
                    Route::get('edit_unique/{id}', [UserController::class, 'edit_unique'])->name('user_unique_edit');
                    Route::post('update_unique/{id}', [UserController::class, 'update_unique'])->name('user_unique_update');
                });
                Route::group(['prefix' => 'profile'], function () {
                    Route::get('/', [ProfileController::class, 'index'])->name('profile_index');
                    Route::get('create', [ProfileController::class, 'create'])->name('profile_create');
                    Route::post('store', [ProfileController::class, 'store'])->name('profile.store');
                    Route::get('edit/{id}', [ProfileController::class, 'edit'])->name('profile.edit');
                    Route::post('update', [ProfileController::class, 'update'])->name('profile.update');
                    Route::get('getData', [ProfileController::class, 'getData'])->name('profile.data');
                    Route::get('view/{id}', [ProfileController::class, 'show'])->name('profile.view');
                    Route::get('delete/{id}', [ProfileController::class, 'destroy'])->name('profile.delete');
                });
            });

            Route::group(['prefix' => 'productos-secciones'], function () {
                Route::get('/', [productosSeccionesController::class, 'index'])->name('productosSecciones.index');
                Route::get('create', [productosSeccionesController::class, 'create'])->name('productosSecciones.create');
                Route::post('store', [productosSeccionesController::class, 'store'])->name('productosSecciones.store');
                Route::get('edit/{id}', [productosSeccionesController::class, 'edit'])->name('productosSecciones.edit');
                Route::post('update/{id}', [productosSeccionesController::class, 'update'])->name('productosSecciones.update');
                Route::get('delete/{id}', [productosSeccionesController::class, 'destroy'])->name('productosSecciones.delete');
            });

            Route::group(['prefix' => 'banners-home'], function () {
                Route::get('/', [bannersController::class, 'index'])->name('banners.index');
                Route::get('create', [bannersController::class, 'create'])->name('banners.create');
                Route::post('store', [bannersController::class, 'store'])->name('banners.store');
                Route::get('edit/{id}', [bannersController::class, 'edit'])->name('banners.edit');
                Route::post('update/{id}', [bannersController::class, 'update'])->name('banners.update');
                Route::get('delete/{id}', [bannersController::class, 'destroy'])->name('banners.delete');
            });
        });
    });
});

Route::get('/', function () {
    return redirect("/cms");
});

/**********/
/* LOGOUT */
/**********/
Route::get('/logout', function() {
    Auth::logout();
    Session::flush();
    return redirect('/');
});

/***********/
/* MAILING */
/***********/
Route::post('/sendContacto', 'SendController@send');

/********/
/* TEST */
/********/

Route::get('/clear-cache', function() {
    $exitCode = Artisan::call('cache:clear');
    $exitCode = Artisan::call('config:cache');
    return 'DONE'; //Return anything
});